"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Router = require("koa-router");
const settings_1 = require("../../settings");
const form_1 = require("../helper/form");
const table_1 = require("../helper/table");
const errors_1 = require("../helper/errors");
const database_1 = require("../../database");
const core_1 = require("@rtdb2/core");
const config_1 = require("../../config");
const hb_1 = require("../helper/hb");
const AdminRoute = new Router();
AdminRoute.use(async (ctx, next) => {
    const { key } = ctx.query;
    if (key !== config_1.default.admin)
        throw new errors_1.NoPermissionError("No permission!");
    return next();
});
AdminRoute.get("/", async (ctx) => {
    //TODO: Main Interface
    ctx.body = (0, hb_1.getView)("admin");
});
AdminRoute.get("/settings", async (ctx) => {
    let res = await new Promise((yes, no) => {
        const stream = settings_1.default.db.createReadStream({
            keys: true,
            values: true,
            valueAsBuffer: true,
        });
        let res = [["key", "value"]];
        stream.on("data", ({ key, value }) => {
            res.push([key, value]);
        });
        stream.on("error", no);
        stream.on("end", () => yes(res));
    });
    if (ctx.query.view) {
        return (0, table_1.default)("Settings", res, ctx);
    }
    else {
        ctx.body = res;
    }
});
AdminRoute.get("/data", async (ctx) => {
    const { database } = ctx.query;
    let db = database_1.DatabaseManager.getDatabase(database);
    if (!db)
        throw new errors_1.BadRequestError("Database not found");
    let res = await new Promise((yes, no) => {
        const stream = db.database.data.createReadStream({
            keys: true,
            values: true,
            valueAsBuffer: true,
            keyAsBuffer: false,
            limit: 1000,
        });
        let res = [["key", "value"]];
        stream.on("data", ({ key, value }) => {
            res.push([
                key,
                key.split("/").length > 2
                    ? value.toString()
                    : JSON.stringify(core_1.MP.decode(value)),
            ]);
        });
        stream.on("error", no);
        stream.on("end", () => yes(res));
    });
    if (ctx.query.view) {
        return (0, table_1.default)("Data from " + database, res, ctx);
    }
    else {
        ctx.body = res;
    }
});
AdminRoute.get("/database", (ctx) => {
    const isFull = ctx.query.full === "true" || ctx.query.full === "1";
    let res;
    if (isFull) {
        //TODO: Better than JSON.parse / JSON.stringify
        res = Array.from(database_1.DatabaseManager.databases.entries()).map(([name, config]) => ({
            name,
            ...JSON.parse(JSON.stringify(config)),
        }));
    }
    else {
        res = Array.from(database_1.DatabaseManager.databases.keys());
    }
    if (ctx.query.view) {
        return (0, table_1.default)("Databases" + (isFull ? "" : " small"), res, ctx);
    }
    else {
        ctx.body = res;
    }
}).post("/database", async (ctx) => {
    const { name, rules, publickey, accesskey, rootkey } = ctx.request.body;
    if (!name)
        throw new errors_1.BadRequestError("Name must be set!");
    let db = database_1.DatabaseManager.getDatabase(name);
    if (!db)
        db = await database_1.DatabaseManager.addDatabase(name);
    if (publickey)
        await db.setPublicKey(publickey);
    if (rules) {
        const error = await db.setRules(rules);
        if (error) {
            ctx.status = 500;
            ctx.body = error;
            return;
        }
    }
    if (accesskey)
        await db.setAccessKey(accesskey);
    if (rootkey)
        await db.setRootKey(rootkey);
    ctx.body = "Success";
});
AdminRoute.get("/collections", async (ctx) => {
    const { database } = ctx.query;
    let db = database_1.DatabaseManager.getDatabase(database);
    if (!db)
        throw new errors_1.BadRequestError("Database not found");
    let res = await new Promise((yes, no) => {
        const stream = db.database.collections.createKeyStream({
            keyAsBuffer: false,
            limit: 1000,
        });
        let res = [];
        stream.on("data", (key) => {
            res.push(key);
        });
        stream.on("error", no);
        stream.on("end", () => yes(res));
    });
    if (ctx.query.view) {
        return (0, table_1.default)("Databases", res, ctx);
    }
    else {
        ctx.body = res;
    }
});
AdminRoute.get("/collections/cleanup", async (ctx) => {
    const { database } = ctx.query;
    let db = database_1.DatabaseManager.getDatabase(database);
    if (!db)
        throw new errors_1.BadRequestError("Database not found");
    let deleted = await db.database.runCleanup();
    if (ctx.query.view) {
        return (0, table_1.default)("Databases", deleted, ctx);
    }
    else {
        ctx.body = deleted;
    }
});
AdminRoute.get("/database/new", (0, form_1.default)("/v1/admin/database", "New Database", {
    name: { label: "Name", type: "text" },
    accesskey: { label: "Access Key", type: "text" },
    rootkey: { label: "Root access key", type: "text" },
    rules: {
        label: "Rules",
        type: "codemirror",
        value: `{\n   ".write": true, \n   ".read": true \n}`,
    },
    publickey: { label: "Public Key", type: "textarea" },
}));
AdminRoute.get("/database/update", async (ctx) => {
    const { database } = ctx.query;
    let db = database_1.DatabaseManager.getDatabase(database);
    if (!db)
        throw new errors_1.NotFoundError("Database not found!");
    (0, form_1.default)("/v1/admin/database", "Change Database", {
        name: {
            label: "Name",
            type: "text",
            value: db.name,
            disabled: true,
        },
        accesskey: {
            label: "Access Key",
            type: "text",
            value: db.accesskey,
        },
        rootkey: {
            label: "Root access key",
            type: "text",
            value: db.rootkey,
        },
        rules: {
            label: "Rules",
            type: "codemirror",
            value: db.rules,
        },
        publickey: {
            label: "Public Key",
            type: "textarea",
            value: db.publickey,
        },
    })(ctx);
});
exports.default = AdminRoute;
//# sourceMappingURL=admin.js.map