"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebsocketConnectionManager = void 0;
const logging_1 = require("@hibas123/logging");
const WebSocket = require("ws");
const database_1 = require("./database");
const core_1 = require("@rtdb2/core");
const jwt_1 = require("./helper/jwt");
const nanoid_1 = require("nanoid");
class WebsocketConnectionManager {
    static bind(server) {
        this.server = new WebSocket.Server({ server });
        this.server.on("connection", this.onConnection.bind(this));
    }
    static async onConnection(socket, req) {
        logging_1.default.log("New Connection:");
        socket.on("error", (err) => {
            logging_1.default.error(err);
            socket.close();
        });
        const sendError = (error) => socket.send(JSON.stringify({ ns: "error_msg", data: error }));
        const session = new core_1.Session((0, nanoid_1.nanoid)());
        const query = new URL(req.url, "http://localhost").searchParams;
        const database = query.get("database");
        const db = database_1.DatabaseManager.getDatabase(database);
        if (!db) {
            sendError("Invalid Database!");
            socket.close();
            return;
        }
        const accesskey = query.get("accesskey");
        if (db.accesskey) {
            if (!accesskey || accesskey !== db.accesskey) {
                sendError("Unauthorized!");
                socket.close();
                return;
            }
        }
        const authkey = query.get("authkey");
        if (authkey && db.publickey) {
            let res = await (0, jwt_1.verifyJWT)(authkey, db.publickey);
            if (res && !res.uid && res.user)
                res.uid = res.user;
            if (!res || !res.uid) {
                sendError("Invalid JWT");
                socket.close();
                return;
            }
            else {
                session.uid = res.uid;
            }
        }
        const rootkey = query.get("rootkey");
        if (rootkey && db.rootkey) {
            if (rootkey === db.rootkey) {
                session.root = true;
                logging_1.default.warning(`Somebody logged into ${database} via rootkey`);
            }
        }
        const answer = (id, data, err) => {
            let error = false;
            if (err) {
                logging_1.default.error(err);
                if (err instanceof Error) {
                    data = err.message;
                }
                else {
                    data = err;
                }
                error = true;
            }
            socket.send(JSON.stringify({ ns: "message", data: { id, error, data } }));
        };
        const handler = new Map();
        handler.set("v2", async ({ id, query }) => db.database
            .run(Array.isArray(query) ? query : [query], session)
            .then((res) => answer(id, res))
            .catch((err) => answer(id, undefined, err)));
        // handler.set("bulk", async ({ id, query }) => db.run(query, session)
        //    .then(res => answer(id, res))
        //    .catch(err => answer(id, undefined, err))
        // );
        const SnapshotMap = new Map();
        handler.set("snapshot", async ({ id, query, }) => {
            db.database
                .snapshot(query, session, (data) => {
                logging_1.default.debug("Sending snapshot");
                socket.send(JSON.stringify({
                    ns: "snapshot",
                    data: { id, data },
                }));
            })
                .then((s) => {
                answer(id, s.snaphot);
                SnapshotMap.set(id, s.id);
            })
                .catch((err) => answer(id, undefined, err));
        });
        handler.set("unsubscribe", async ({ id }) => {
            let i = SnapshotMap.get(id);
            if (i) {
                db.database.unsubscribe(i, session);
                SnapshotMap.delete(i);
            }
        });
        socket.on("message", async (rawData) => {
            try {
                let message = JSON.parse(rawData);
                let h = handler.get(message.ns);
                if (h) {
                    h(message.data);
                }
            }
            catch (err) {
                logging_1.default.error("Unknown Error:");
                logging_1.default.error(err);
            }
        });
        db.database.connectionCount++;
        socket.on("close", () => {
            db.database.connectionCount--;
            logging_1.default.log(`${session.id} has disconnected!`);
            session.subscriptions.forEach((unsubscribe) => unsubscribe());
            session.subscriptions.clear();
            socket.removeAllListeners();
        });
    }
}
exports.WebsocketConnectionManager = WebsocketConnectionManager;
//# sourceMappingURL=websocket.js.map